""" Specify version information about GSH.

This file is meant to be kept minimal. It is loaded by both GSH itself
and setup.py. This is to avoid having to specify the code in multiple
places. Because of this, this file should remain empty other than the
__version__ itself.
"""

__version__ = "0.10.1"
